/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef __DT_SOUND_QCOM_WCD934x_H
#define __DT_SOUND_QCOM_WCD934x_H

#define AIF1_PB                 0
#define AIF1_CAP                1
#define AIF2_PB                 2
#define AIF2_CAP                3
#define AIF3_PB                 4
#define AIF3_CAP                5
#define AIF4_PB                 6
#define AIF4_VIFEED             7
#define AIF4_MAD_TX             8

#endif
