/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_POWER_SUN8I_V853_PPU_H_
#define _DT_BINDINGS_POWER_SUN8I_V853_PPU_H_

#define PD_RISCV		0
#define PD_NPU			1
#define PD_VE			2

#endif
