/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * This header provides constants for the defined MHUv3 types.
 */

#ifndef _DT_BINDINGS_ARM_MHUV3_DT_H
#define _DT_BINDINGS_ARM_MHUV3_DT_H

#define DBE_EXT		0
#define FCE_EXT		1
#define FE_EXT		2

#endif /* _DT_BINDINGS_ARM_MHUV3_DT_H */
