/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2019, The Linux Foundation. All rights reserved.
 * Copyright (c) 2023, Linaro Limited
 */

#ifndef _DT_BINDINGS_RESET_QCOM_VIDEO_CC_SM8350_H
#define _DT_BINDINGS_RESET_QCOM_VIDEO_CC_SM8350_H

#define VIDEO_CC_CVP_INTERFACE_BCR				0
#define VIDEO_CC_CVP_MVS0_BCR					1
#define VIDEO_CC_MVS0C_CLK_ARES					2
#define VIDEO_CC_CVP_MVS0C_BCR					3
#define VIDEO_CC_CVP_MVS1_BCR					4
#define VIDEO_CC_MVS1C_CLK_ARES					5
#define VIDEO_CC_CVP_MVS1C_BCR					6

#endif
