/* SPDX-License-Identifier: GPL-2.0 */
/*
 * NFSv3 protocol definitions
 */
#ifndef _LINUX_NFS3_H
#define _LINUX_NFS3_H

#include <uapi/linux/nfs3.h>


/* Number of 32bit words in post_op_attr */
#define NFS3_POST_OP_ATTR_WORDS		22

#endif /* _LINUX_NFS3_H */
