// SPDX-License-Identifier: (GPL-2.0+ OR MIT)
/*
 * Actions Semi S700 SPS
 *
 * Copyright (c) 2017 Andreas Färber
 */
#ifndef DT_BINDINGS_POWER_OWL_S700_POWERGATE_H
#define DT_BINDINGS_POWER_OWL_S700_POWERGATE_H

#define S700_PD_VDE	0
#define S700_PD_VCE_SI	1
#define S700_PD_USB2_1	2
#define S700_PD_HDE	3
#define S700_PD_DMA	4
#define S700_PD_DS	5
#define S700_PD_USB3	6
#define S700_PD_USB2_0	7

#endif
